/**
 * Key interface with helper methods for parsing types
 */
export interface IKey {
    get(): Promise<string | void | null>;
    set(value: string): Promise<void>;
    delete(): Promise<void>;
    getBoolean(): Promise<boolean>;
    setBoolean(value: boolean): Promise<void>;
    getNumber(): Promise<number>;
    setNumber(value: number): Promise<void>;
    getDate(): Promise<Date>;
    setDate(value: Date): Promise<void>;
    getJSON<T>(): Promise<T>;
    setJSON(value: any): Promise<void>;
}
declare type KeyId = string;
/**
 * Abstract class to implement your keys.
 * Keys can be powered by any storage system!
 *
 * Only need to override `get`, `set`, `delete`.
 */
export declare abstract class Key<T extends KeyId> implements IKey {
    name: T;
    constructor(name: T);
    abstract get(): Promise<string | null>;
    abstract set(value: string): Promise<void>;
    abstract delete(): Promise<void>;
    getBoolean(): Promise<boolean>;
    setBoolean(value: boolean): Promise<void>;
    getNumber(): Promise<number>;
    setNumber(value: number): Promise<void>;
    getDate(): Promise<Date>;
    setDate(value: Date): Promise<void>;
    getJSON<T>(): Promise<T>;
    setJSON(value: any): Promise<void>;
}
/**
 * Type for your plain `storage` object
 */
export declare type TypedStorage<T extends KeyId> = Record<T, Key<T>>;
/**
 * Interface for convenience storage object
 */
export interface IStorage<T extends KeyId> {
    /** Use a key in storage */
    use(key: T): Key<T>;
    /**
     * Delete all keys in storage
     */
    clear(): Promise<void>;
    /** get storage as json object */
    getJSON(): Promise<Record<T, string | undefined | null>>;
}
/**
 * Convenience storage object that wraps `TypedStorage` to provide
 * common functionalities like using a key, clearing whole storage,
 * getting storage as json, etc.
 *
 * You don't have to use this. You can instead use the `TypedStorage` storage directly
 * or create your own abstraction!
 */
export declare class JSStorage<T extends KeyId> implements IStorage<T> {
    private storage;
    constructor(storage: TypedStorage<T>);
    use(key: T): Key<T>;
    clear(): Promise<void>;
    getJSON(): Promise<Record<T, string | undefined | null>>;
}
export {};
